//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	ColumnSelector.h:	--- a Qt widget with a list of selectable table columns
//
//	DESCRIPTION:
//		A new Qt widget class is declared - details to follow
//
//	CHANGE HISTORY:
//		UL, 1998-10-22: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Thu Oct 22 14:15:14 MET DST 1998
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/ColumnSelector.h,v 1.12 2002/02/06 16:30:13 ulammers Exp $
//

#ifndef COLUMN_SELECTOR_H
#define COLUMN_SELECTOR_H

#include "TableInquiry.h"

#include <strstream>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qframe.h>
#include <string>
#include <qwidget.h>

#include "XmmSelectServer.h"

class ExpressionConstructor;


class SelectionGroup : public QFrame {
	Q_OBJECT

public:
	SelectionGroup(QWidget *parent, const string &colName);

	QPushButton *twoDprojectionCheck() const { return _twoDprojectionCheck; }
	QRadioButton *oneDprojectionCheck() const { return _oneDprojectionCheck; }

	const string &name() const { return _colName; }

	virtual ~SelectionGroup();

protected:
	QGridLayout *setupCheckButtons(bool twoDprojCheck, bool oneDprojCheck);

private:
	const string _colName;
	QPushButton	 *_twoDprojectionCheck;
	QRadioButton *_oneDprojectionCheck;
}; // SelectionGroup


class ColumnSelector : public QWidget {
	Q_OBJECT

public:
	ColumnSelector(QWidget *parent, const char *name,
				   ExpressionConstructor *exprConstructor,
				   const vector<TableColumn> &tableColumns);

	const TableColumn *twoDprojectionAxisX() const;
	double twoDprojectionAxisXmin() const;

	const TableColumn *twoDprojectionAxisY() const;
	double twoDprojectionAxisYmin() const;

	const TableColumn *oneDprojectionAxis() const;
	double oneDprojectionAxisMin() const;

	void addColumn(const TableColumn &tableColumn);

	~ColumnSelector();

private slots:
	void booleanSelection(const string &, bool, bool);
	void stringSelection(const string &, const string &, bool);
	void integerSelection(const string &, int, int, int, int, QPushButton *);
	void realSelection(const string &, double, double, bool, bool,
					   QPushButton *);

	void twoDprojectionAxisSelected(int);
	void oneDprojectionAxisSelected(int);

private:
	//	data members
	ExpressionConstructor	*_exprConstructor;
	QButtonGroup			*_twoDprojectionButtons, *_oneDprojectionButtons;
	QBoxLayout				*_layout;
	QGridLayout				*_grid;

	const vector<TableColumn> &_tableColumns;
	int				_twoDprojectionCol[2];
	unsigned		_nextIndexToChange;
	int				_oneDprojectionCol;

	//	methods
	SelectionGroup *addColumn(QWidget *parent,
							  const TableColumn &tableColumn);
	void registerColumn(SelectionGroup *widget, int id);

	template<class Type>
	void minGreaterThanMax(Type min, Type max, QPushButton *p = 0) {
		strstream s;
		s<<"The lower bound ("<<min<<") is greater than the upper\n"
		   "bound ("<<max<<") - that does not make sense."<<ends;
		string st(s.str());
		QMessageBox::warning(xmmSelectServer->gui(),
							 "Min>Max", st.c_str(), "Ok");
		if (p)
			p->setDown(false);
	}
}; // ColumnSelector


class IntegerSelectionGroup : public SelectionGroup {
	Q_OBJECT

public:
	IntegerSelectionGroup(QWidget *parent, const TableColumn &tableColumn);
	~IntegerSelectionGroup();

private slots:
	void signalLimits();

signals:
	void selected(const string &, int, int, int, int, QPushButton *);

private:
	QGridLayout	*_grid;
	QPushButton	*_columnButton;
	QSpinBox	*_minSpinBox, *_maxSpinBox;
	int			_colMinVal, _colMaxVal;
}; // IntegerSelectorGroup


class RealSelectionGroup : public SelectionGroup {
	Q_OBJECT

public:
	RealSelectionGroup(QWidget *parent, const TableColumn &tableColumn);
	~RealSelectionGroup();

private slots:
	void signalLimits();
	void minChanged();
	void maxChanged();

signals:
	void selected(const string &, double, double, bool, bool, QPushButton *);

private:
	QGridLayout	*_grid;
	QPushButton	*_columnButton;
	QLineEdit	*_maxLineEdit, *_minLineEdit;
	bool		_minChanged, _maxChanged;
}; // RealSelectorGroup


class StringSelectionGroup : public SelectionGroup {
	Q_OBJECT

public:
	StringSelectionGroup(QWidget *parent, const TableColumn &tableColumn);
	~StringSelectionGroup();

private slots:
	void signalValue();
	void valueChanged();

signals:
	void selected(const string &, const string &, bool);

private:
	QGridLayout		*_grid;
	QLineEdit		*_valueLineEdit;
	bool			_valueChanged;
}; // StringSelectorGroup


class BooleanSelectionGroup : public SelectionGroup {
	Q_OBJECT

public:
	BooleanSelectionGroup(QWidget *parent, const TableColumn &tableColumn);
	~BooleanSelectionGroup();

private slots:
	void valueChanged(int);
	void signalValue();

signals:
	void selected(const string &, bool, bool);

private:
	QGridLayout		*_grid;
	QButtonGroup	*_valueButton;
	bool			_value;
	bool			_valueChanged;
}; // BooleanSelectorGroup

#endif
