//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	TableInquiry.h --- classes describing table structure
//
//	DESCRIPTION:
//		Classes related to Dal table columns
//
//	CHANGE HISTORY:
//		UL, 1998-10-22: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Thu Oct 22 14:15:14 MET DST 1998
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/TableInquiry.h,v 1.2 2002/02/06 16:30:13 ulammers Exp $
//

#ifndef TABLE_INQUIRY_H
#define TABLE_INQUIRY_H

#include <string>

#include "Tracer.h"
#include "Dal.h"

//
//	---( TableColumn )--------------------------------------------------------
//
class TableColumn {
public:
	TableColumn(const string &name, Dal::DataType type)
		: _name(name), _type(type),
		  _dMin(0.), _dMax(0.) { }
	TableColumn(const Column *column);

	const string &name() const { return _name; }
	Dal::DataType type() const { return _type; }
	static string toString(Dal::DataType);

	double dmin() const { return _dMin; }
	double dmax() const { return _dMax; }

	~TableColumn() { }

private:
	string			_name;
	Dal::DataType	_type;
	double			_dMin, _dMax;
}; // TableColumn


//
//	---( TableInquiry )------------------------------------------------------
//
class TableInquiry : public vector<TableColumn> {
public:
	TableInquiry(const string &tableName);

	Table *openTable() const;
	void  closeTable() const;
	const string &name() const { return _name; }
	const string &dssSelections() const { return _dssSelections; }

	~TableInquiry() { }

private:
	string	_name,
			_dssSelections;
	mutable Table	*_table;
}; // TableInquiry

#endif

