//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	TaskLauncher.h:	--- classes related to launching tasks
//
//	DESCRIPTION:
//		Classes related to launching tasks
//
//	CHANGE HISTORY:
//		UL, 1998-10-22: 1.0.0: - first version
//		UL, 1999-01-29: 1.1.0: - minor fix to make it build under egcs
//		UL, 2000-05-09: 1.2.0: - minor fix
//
//	AUTHOR:
//		U. Lammers (UL), Thu Oct 22 14:15:14 MET DST 1998
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/TaskLauncher.h,v 1.17 2002/02/06 16:30:13 ulammers Exp $
//

#ifndef TASK_LAUNCHER_H
#define TASK_LAUNCHER_H

#include <string>
#include <vector>

#include <Grace.h>
#undef DEBUG
#include "Tracer.h"
#include "TableInquiry.h"
#include "ExceptionErrorHandler.h"
#include "ExprTranslator.h"

#include <qapplication.h>

class TaskLauncher : public QObject {
	Q_OBJECT

public:
	TaskLauncher(const string &name);

	const string &name() const { return _name; }

	~TaskLauncher();

private:
	string _name;
};


class EvselectLauncher : public TaskLauncher {
	Q_OBJECT

public:
	EvselectLauncher();

	double oneDbin() const { return _oneDbin; }
	double oneDmin() const { return _oneDmin; }
	bool withOneDmin() const { return _withOneDmin; }

	double binX() const { return _binX; }
	double binY() const { return _binY; }
	double minX() const { return _minX; }
	double minY() const { return _minY; }

	bool withMinX() const { return _withMinX; }
	bool withMinY() const { return _withMinY; }
	bool quicklook() const { return _quicklook; }
	bool withBackscaleDiag() const { return _withBackscaleDiag; }

	~EvselectLauncher();

public slots:
	bool filterTable(const string &dsName, const string &expression,
					 const string &filteredSet);
	bool extractImageAndDisplay(
						 const string &dsName, const string &expression,
						 const TableColumn *xColumn,
						 const TableColumn *yColumn);
	bool extractSpectrumAndDisplay(
						 const string &dsName, const string &expression,
						 const TableColumn *column,
						 bool runBackscale);
	bool extractRatesCurveAndDisplay(
						 const string &dsName, const string &expression,
						 const TableColumn *column);
	bool extractHistogramAndDisplay(
						 const string &dsName, const string &expression,
						 const TableColumn *column);

private:
	bool isValidExpression(const string &expression);
		//	true if expression is syntactically correct
	bool run();
		//	true if evselect ran successfully

	bool	_quicklook, _withBackscaleDiag;
	double	_binX, _binY,
			_minX, _minY;
	bool	_withMinX, _withMinY;

	double	_oneDmin, _oneDbin;
	bool	_withOneDmin;
}; // EvselectLauncher


class ProductViewer {
public:
	ProductViewer() {}
	virtual ~ProductViewer() {}

	virtual vector<string> getRegion() const = 0;
	virtual RegionSpecification *regionSpec(const string &s) const = 0;
}; // ProductViewer


class ImageDisplayLauncher : public TaskLauncher, public ProductViewer {
	Q_OBJECT

public:
	ImageDisplayLauncher();
	~ImageDisplayLauncher();

	RegionSpecification *regionSpec(const string &s) const;

public slots:
	void displayImage(const string &);
	vector<string> getRegion() const;
}; // ImageDisplayLauncher


class DsplotLauncher : public TaskLauncher, public ProductViewer {
	Q_OBJECT

public:
	DsplotLauncher();
	~DsplotLauncher();

	RegionSpecification *regionSpec(const string &s) const;

	void down() const;

	//
	//	set/define offset value
	//
	double offsetX() const;

	void displayDataSet(const string &, const string &, const string &,
						bool scatter = false, bool offsetX = false);
	vector<string> getRegion() const;

private:
	void pollRegion(vector<string> &) const;

	static Grace*	_grace;
	static double	_offsetX;
}; // DsplotLauncher

#endif

