//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//				       (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	XmmSelectServer --- Central task server class
//
//	DESCRIPTION:
//		A central task resource server class is declared.
//
//	CHANGE HISTORY:
//		UL, 2000-10-17: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers (UL), Wed Oct 18 10:55:23 MET DST 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/XmmSelectServer.h,v 1.2 2002/02/06 16:30:13 ulammers Exp $
//
#ifndef XMMSELECT_SERVER_H
#define XMMSELECT_SERVER_H

#include <qobject.h>
#include "Tracer.h"
#include "ParameterInfo.h"
#include "Preferences.h"
#include "TableInquiry.h"
#include "XmmSelectGui.h"

class XmmSelectServer : public QObject {
	Q_OBJECT

public:
	XmmSelectServer();

	XmmSelectGui	*gui() const { return _gui; }
	ParameterInfo	&parameters() const { return _myPars; }
	PreferenceGroup *preferences() const { return _prefs; }
	TableInquiry	*tableInquiry() const { return _tableInquiry; }

	~XmmSelectServer();

	static XmmSelectServer *instance();

public slots:
	void reloadTable();
	void newTable(string tableName);

signals:
	void tableChanged();

private:
	static XmmSelectServer *_instance;
	XmmSelectGui	*_gui;
	PreferenceGroup	*_prefs;
	TableInquiry	*_tableInquiry;
	ParameterInfo	&_myPars;
};

#define xmmSelectServer XmmSelectServer::instance()

#endif
